/**
 * Created by kevinries on 04.07.16.
 */
var classify = require('../util/lang').classify;
var mergeOptions = require('../util/options').mergeOptions;
module.exports = function (F3) {

    F3.options = {
        components: {},
        replace: true
    }

    F3.cid = 0
    var cid = 1

    /**
     * Class inheritance
     *
     * @param {Object} extendOptions
     */

    F3.extend = function (extendOptions) {
        extendOptions = extendOptions || {}
        var Super = this
        var isFirstExtend = Super.cid === 0
        if (isFirstExtend && extendOptions._Ctor) {
            return extendOptions._Ctor
        }
        var name = extendOptions.name || Super.options.name
        if (process.env.NODE_ENV !== 'production') {
            if (!/^[a-zA-Z][\w-]*$/.test(name)) {
                warn(
                    'Invalid component name: "' + name + '". Component names ' +
                    'can only contain alphanumeric characaters and the hyphen.'
                )
                name = null
            }
        }
        var Sub = createClass(name || 'F3Component')
        Sub.prototype = Object.create(Super.prototype)
        Sub.prototype.constructor = Sub
        Sub.cid = cid++
        Sub.options = mergeOptions(
            Super.options,
            extendOptions
        )
        Sub['super'] = Super
        // allow further extension
        Sub.extend = Super.extend
        // create asset registers, so extended classes
        // can have their private assets too.

        // enable recursive self-lookup
        if (name) {
            Sub.options.components[name] = Sub
        }
        // cache constructor
        if (isFirstExtend) {
            extendOptions._Ctor = Sub
        }
        return Sub
    }

    /**
     * A function that returns a sub-class constructor with the
     * given name. This gives us much nicer output when
     * logging instances in the console.
     *
     * @param {String} name
     * @return {Function}
     */

    function createClass (name) {
        /* eslint-disable no-new-func */
        return new Function(
            'return function ' + classify(name) +
            ' (options) { this._init(options) }'
        )()
        /* eslint-enable no-new-func */
    }

    /**
     * Plugin system
     * @param plugin
     */
    F3.use = function (plugin) {
        if (plugin.installed) {
            return
        }
        // additional parameters
        var args = toArray(arguments, 1)
        args.unshift(this)
        if (typeof plugin.install === 'function') {
            plugin.install.apply(plugin, args)
        } else {
            plugin.apply(null, args)
        }
        plugin.installed = true
        return this
    }
}