/**
 * Created by kevinries on 04.07.16.
 */
var F3 = require('../instance/f3');
var langUtil = require('./lang');

var strategies = Object.create(null);

strategies.created =
    strategies.init =
    strategies.activated =
    function (parentVal, childVal) {
        return childVal
            ? parentVal
            ? parentVal.concat(childVal)
            : Array.isArray(childVal)
            ? childVal
            : [childVal]
            : parentVal
}

var defaultStrat = function (parentVal, childVal) {
    return childVal === undefined
        ? parentVal
        : childVal
}


function guardComponents (options) {
    if (options.components) {
        var components = options.components;
        var ids = Object.keys(components)
        var def
        if (process.env.NODE_ENV !== 'production') {
            var map = options._componentNameMap = {}
        }
        for (var i = 0, l = ids.length; i < l; i++) {
            var key = ids[i]
            // record a all lowercase <-> kebab-case mapping for
            // possible custom element case error warning
            if (process.env.NODE_ENV !== 'production') {
                map[key.replace(/-/g, '').toLowerCase()] = langUtil.hyphenate(key)
            }
            def = components[key]
            if (langUtil.isPlainObject(def)) {
                components[key] = F3.extend(def)
            }
        }
    }
}

module.exports.mergeOptions = function(parent,child,vm){
    guardComponents(child);
    var options = {};
    var key;
    if (child.extends) {
        parent = typeof child.extends === 'function'
            ? mergeOptions(parent, child.extends.options, vm)
            : mergeOptions(parent, child.extends, vm)
    }
    for(key in parent){
        mergeField(key);
    }

    for(key in child){
        if(!langUtil.hasOwn(parent,key)){
            mergeField(key);
        }
    }

    function mergeField (key) {
        var strat = strategies[key] || defaultStrat
        options[key] = strat(parent[key], child[key], vm, key)
    }

    return options;
}

module.exports.resolveAsset = (options, type, id, warnMissing) => {
    /* istanbul ignore if */
    if (typeof id !== 'string') {
        return
    }
    var assets = options[type]
    var camelizedId
    var res = assets[id] ||
        // camelCase ID
        assets[camelizedId = langUtil.camelize(id)] ||
        // Pascal Case ID
        assets[camelizedId.charAt(0).toUpperCase() + camelizedId.slice(1)]
    if (process.env.NODE_ENV !== 'production' && warnMissing && !res) {
        warn(
            'Failed to resolve ' + type.slice(0, -1) + ': ' + id,
            options
        )
    }
    return res
}